/** LOCATION SEARCH - Met Office specific Jquery Plugin
* Author: Josh Wreford
* Plugin transforms a text box into an autocompleting location search using files collectively containing 250,000 locations across the UK 
* As soon as a user has typed 3 letters, the script requests a text file containing all the locations beginning with those letters.
* The data is then cached and if subsequent letters are entered into the box, the cached data is queried so that it only returns locations containing all entered characters. 
* When a location is selected a callback function is fired with the selected object as an argument (object values are name, lat, lng, county)
*/
(function($){
	$.fn.locsearch = function(callback, useLocations, useRegions, usePostcodes, useMountains, rootURL){
		
		var thisBox = this;
		var typeArray = []; 
		var urlRoot = "";
		if(typeof rootURL !== "undefined" && rootURL != null && rootURL.length > 0){
			urlRoot = rootURL;
		}
		
		/*
		 * Attempts to translate the specified text. This will check for the existence of the _()
		 * method provided by the MetOfficeLang.js library. If it exists the translation is
		 * fetched via _(text). The JQuery autocomplete functionality uses the text() method that
		 * ensures that the passed in text is rendered exactly and will escape any escape chars. This
		 * causes issues when handling accented names which use escape codes in the format &#224;. The
		 * text() method will render the escape code rather than the accent eg "A' &#224;e". Therefore
		 * an additional fix has been added that checks for the existance of the &# code and parses the
		 * text to replace any escape codes with the actual accented chars so that when the auto complete
		 * code passes the text to the text() method the correct accents are already present.
		 * 
		 * This additional functionality is controlled by setting the optional unescape boolean to true
		 * and is currently only used when displaying the county/region associated with a site.
		 */
        var translate = function(text, unescape){
          if(typeof _ == 'function')
          {
            var val = _(text);
            if(unescape!='undefined' && unescape && val.indexOf('&#')>=0)
            {
              val = $('<div/>').html(val).text();
            }
            return val;
          }
          return text;
        };
        
        function isAPostcode(text){
            if((typeof metoffice) != 'undefined' && (typeof metoffice.utility) !='undefined' &&
            		(typeof metoffice.utility.isAPostcode) == 'function')
            {
            	return metoffice.utility.isAPostcode(text);
            }
            else if((typeof metOffice) != 'undefined' && (typeof metOffice.location) !='undefined' && 
            		(typeof metOffice.location.isAPostcode) == 'function'){
            	return metOffice.location.isAPostcode(text);
            }
            else{
            	return false;
            }
        }
  
        typeArray[1] = translate("Location Forecasts");
        typeArray[2] = translate("Mountain Forecasts");
        typeArray[3] = translate("Regional Forecasts");
		var suggestions = [], cache = {};
		//sorts results by type
		var sortByType = function(a,b) {
			if (a.t > b.t) {return -1;}
			if (a.t < b.t) {return 1;}
			return 0;
		};
		/*var onEnter = function() {
			if(event.keyCode==13) {
				alert("return key hit");
			}
		}*/
		//extends jquery autocomplete to add in categories to group results into type
		$.widget( "custom.autocompleteByType", $.ui.autocomplete, {
			_renderMenu: function( ul, items ) {
				var self = this, currentType = "";
				if ((items.length == 1) && (items[0].value == "No results")) {
					ul.append( "<li class='ui-autocomplete-category'>"+translate("No results found")+"</li>" );
					self._renderItem( ul, "" );
				} else {
					items.sort(sortByType);
					$.each( items, function( index, item ) {
						if ( item.t != currentType ) {
							ul.append( "<li class='ui-autocomplete-category'>" + typeArray[item.t] + "</li>" );
							currentType = item.t;
						}
						self._renderItem( ul, item );
					});
				}
			}
		});
		$(this).autocompleteByType({
			minLength: 2,
			//define callback to format results
			source: function(req, add){
				suggestions = [];
				var entry = parseSearchTerm(req.term);

				if (usePostcodes && isAPostcode(entry)) {
					//Get Postcode
					if (entry.substring(0,4) in cache) {
						removeLoading();
						callback(cache[entry.substring(0,4)], req);
						hideDropdown();
					} else {
						$.ajax({
							url: "http://www.metoffice.gov.uk/public/data/CoreXMLCache/LatLng/ItemList/Postcode/LatLng?postcode="+(entry.substring(0,4)).toUpperCase(),
							dataType: "xml",
							type: "GET",
							//if data is returned successfully
							success: function (data) {
								//add data to cache to avoid subsequent duplicate requests
								$(data).find('site').each(function(){
									var pName = $(this).attr('name'), pLat = $(this).attr('lat'), pLng = $(this).attr('lng');
									var postcodeSite = {'n':pName,'l':pLat,'g':pLng,'t':1};
									cache[entry.substring(0,4)] = postcodeSite;
									removeLoading();
									hideDropdown();
									callback(postcodeSite);
								});
							},
							error: function (status) {
								/*if no results*/add(["No results"]);
								noResults();
							}
						});
					}
				} else if (entry.length > 2) {
					if (entry.substring(0,3) in cache) {
						addLocations(cache[entry.substring(0,3)], req);
						if (suggestions.length > 0) {
							add(suggestions);
						} else {
							/*if no results*/add(["No results"]);
						}
					} else {
						//Get list of matching locations
						$.ajax({
							url: "http://www.metoffice.gov.uk/public/locationsSearch/locations/"+entry.substring(0,3)+"_.txt",
							dataType: "json",
							method: "GET",
							//if data is returned successfully
							success: function (data, status) {
								//add data to cache to avoid subsequent duplicate requests
								cache[entry.substring(0,3)] = data;
								addLocations(data, req);
								if (suggestions.length > 0) {
									add(suggestions);
								} else {
									/*if no results*/add(["No results"]);
								}
							},
							error: function (status) {
								/*if no results*/add(["No results"]);
								noResults();
							}
						} );
					}
				} else {
					removeLoading();
				}
			//if data is already in local cache
				
			},
			select: function(e, ui) {
				//fire callback function that was specified by the user and pass in the chosen site object
				callback(ui.item, e, ui);
			}
		});  
		
		var parseSearchTerm = function( text){
			var entry = text.toLowerCase();
			while(entry.indexOf(" ")===0){
				entry = entry.substring(1);
			}
			entry = entry.replace("'"," ");
			return entry;
		}
		
		var addLocations = function(data, req) {
			//for each data item, check whether the text input string is contained in the data item string
			var searchTerm = parseSearchTerm(req.term);
		    $.each(data, function(i, val){
				if ((val.t == 1 && useLocations) || (val.t == 2 && useMountains) || (val.t == 3 && useRegions)) {
				   searchName = (val.n).toLowerCase().replace("'"," ");
				   if (searchName.indexOf(searchTerm) != -1) {
						val.label = val.n
						if (val.c) {val.label += " (" + translate(val.c, true) + ")";}
						suggestions.push(val);
					}
				}
			});
		}
		var noResults = function() {
			//TODO: Give the user some sort of feedback that what they have typed has given no results
			removeLoading();
		}
		var removeLoading = function() {
			$(thisBox).removeClass("ui-autocomplete-loading");
		}	
		var hideDropdown = function() {
			$('.ui-autocomplete').css("display","none");
		}
	};
})(jQuery);