/**
 * rss.js
 * 
 * Reads simple RSS feeds. This quite a naive implementation and it assumes that
 * there is a single channel element.
 * 
 * provides:
 *   rss.Feed  - main object represent a single feed (url)
 *   rss.Item  - represents a single Item in a news Feed
 *   rss.Image - data structure for the Feed.image not to be 
 *               confused with the global Image object
 * 
 */
var rss = (function(){	
	// name space private methods
	function accessor(name){
		return function(){
			return this.$node.children(name).text();
		};
	}
	function accessors(){
		var names = Array.prototype.slice.call(arguments,0),
			i,len = names.length,
			result = {};
		for( i = 0 ; i < len ; i ++ ){		
			result[names[i]] = accessor(names[i]);
		}
		return result;
	}
	// return the public prototypes of the namespace
	return {	
		/**
		 * Single Feed item.
		 * constructor:
		 *   new rss.Item(owningFeed, 'item dom node');
		 * properties:
		 *   item.feed - owning feed
		 *   item.guid - guid of the item will be unique in the Feed
		 *   item.read - has this item been marked as read
		 *   
		 * methods:
		 *   item.title() - title of the item
		 *   item.link()  - link to the item
		 *   item.description() - description in html of item 
		 *   item.language() - language of item
		 *   item.pubDate() - string value of the pubication date
		 *   item.enclosure() - {url:'link to some asset', type:'mime type of asset'}
		 *   item.markRead() - mark this item as read will fire the 'itemRead' event in the feed
		 *   
		 */
		Item : (function(){
			// new Item(owningFeed, '<item> dom node');
			var constructor = function(feed, node){
				this.$node = $(node);
				this.feed = feed;
				this.guid = this.$node.children('guid').text();
				this.read = feed.readGuids.indexOf(this.guid) > -1 ;
			};
			constructor.prototype = accessors('title','link','description','language','pubDate');
			constructor.prototype.enclosure = function(){
				var enclosure = this.$node.children('enclosure').eq(0);
				return enclosure.length == 1 ? {
					url:enclosure.attr('url'),
					type:enclosure.attr('type')
				} : null;
			};			
			constructor.prototype.markRead = function(){
				this.feed.markRead(this);
			};
			return constructor;
		})(),
		/**
		 * Image of the RSS feed.
		 * constructor:
		 *  new rss.Image('image dom node')
		 *  
		 * methods:
		 *   item.title() - title of the image
		 *   item.link()  - link to the feed
		 *   item.url() - url of the image 
		 *   item.description() - description of the image 
		 */
		Image : (function(){
			var constructor = function(node){
				this.$node = $(node);				
			};
			constructor.prototype = accessors('title','link','description','url');
			return constructor;
		})(),
		/**
		 * Manages a single RSS feed.
		 * 
		 * constructor:
		 *  new rss.Feed(options);
		 * options:
		 *   url       - location of rss feed
		 *   readGuids - array of item guid that the feed will check for already 
		 *               read items
		 *   load      - function(feed) called after every successful feed.load()
		 *   itemRead  - function(item) call after an item has been marked as read
		 *   unReadItemCountChanged - function(feed,unReadCount,previousValue) 
		 *               called every time the unReadItemCount changes
		 *   error     - function(feed, jqXHR, textStatus, errorThrown) function called if the request
		 *               fails
		 *               
		 * properties: note that these properties will be undefined 
		 *             if the feed hasn't been loaded see (feed.load()) 
		 * 
		 *   feed.title - feed title
		 *   feed.link  - link to the feed 
		 *   feed.description - html description of the over all feed
		 *   feed.language - language of the feed
		 *   feed.pubDate - publication date of the overall feed
		 *   feed.image - will be of rss.Image type              
		 * methods:
		 * 	 feed.load(options) trigger the load/reload of the feed items
		 *                      	success:function(feed) called after successful load
		 *                          error:function(feed,jqXHR, textStatus, errorThrown) 
		 *                          			called on feed load error
		 *   feed.items(function(item)) iterates over the items in the feed, if the feed isn't 
		 *                      loaded then it will call feed.load and iterate once loaded.
		 *                      note if an error occurs this load the call back function will never be called
		 *   feed.unReadItemCount() returns the number of unread items
		 *   
		 */
		Feed : (function(){
			var constructor = function(options){
				this.options = options;
				this.readGuids = options.readGuids || [];
				this._unReadItemCount = 0;
			};
			constructor.prototype = {
					/**
					 * Load the feed.
					 * See rss.Feed level docs for usage.
					 */
					load:function(params){
						var self = this,
							o = this.options,
							added = {};
						params = params || {};
						$.ajax({
							url:o.url,
							method:'GET',
							success:function(data){
								var $doc = $(data);
								/*
								 	<title></title> 
    								<link></link> 
								    <description></description> 
								    <language></language> 
								    <copyright></copyright> 
								    <pubDate></pubDate> 
								 */
								self.title = $doc.find("channel>title").text();
								self.link = $doc.find("channel>link").text();
								self.description = $doc.find("channel>description").text();
								self.language = $doc.find("channel>language").text();
								self.pubDate = $doc.find("channel>pubDate").text();
								self.image = new rss.Image($doc.find("channel>image").get(0));
								$doc.find('item').each(function(){
									var item = new rss.Item(self, this);
									self.addItem(item);
									added[item.guid] = true;
								});
								
								self.trimItems(added);
								if( $.isFunction(o.load)){
									o.load.call(self,self);
								}
								if( $.isFunction(params.success) ){
									params.success.call(self,self);
								}
							},
							error:function(rq, status, error){
								if($.isFunction(o.error)){
									o.error.call(self, self, rq, status, error);
								}
								if($.isFunction(params.error)){
									params.errors.call(self, self, rq, status, error);
								}
							}
						});
					},
					/**
					 * Iterate the items in the feed.
					 * @param callback required function, will be called with the first parameter as each item.
					 */
					items:function(callback){
						var self = this;
						if( this._items ){
							$.each(this._items,function(){callback(this);});
						}else{
							this.load({
								success:function(){
									self.items(callback);
								}
							});
						}
					},
					/**
					 * Accessor for unReadItemCount.
					 * If passed a numeric argument the unReadItemCount will be 
					 * adjusted by that amount firing 'unReadItemCountChanged'.
					 * @return the value of unReadItemCount.
					 */
					unReadItemCount:function(adjustment){
						var previous ;
						if( typeof adjustment === 'number'){
							previous = this._unReadItemCount;
							this._unReadItemCount = previous + adjustment;
							if($.isFunction(this.options.unReadItemCountChanged)){
								this.options.unReadItemCountChanged.call(this,this, this._unReadItemCount, previous);
							}
						}
						return this._unReadItemCount;
					},
					// internal/private(ish) methods from here down
					// not to be called externally
					/**
					 * Implementation of the item.markRead function.
					 * fires the feed 'itemRead' function if present
					 */
					markRead:function(item){
						if( item.read === false){
							item.read = true;
							this.unReadItemCount(-1);
							if( $.isFunction(this.options.itemRead)){
								this.options.itemRead.call(this, item);
							}
						}
					},
					addItem:function(item){
						if( typeof this._items === 'undefined'){
							this._items = [];
							this._itemsByGuid = {};
						}
						// this removes duplicate during refresh
						if( !this._itemsByGuid[item.guid] ){
							this._items.push(item);
							this._itemsByGuid[item.guid] = item;
							if( !item.read ){
								this.unReadItemCount(1);
							}
						}
					},					
					trimItems:function(added){
						var i, guid;
						if( typeof this._items === 'undefined'){
							// do nothing
							return ;
						}
						// iterate backwards.. remove items that are not in 'added'
						for( i = this._items.length-1; i >= 0 ; i -= 1 ){
							guid = this._items[i].guid;
							if( !added[guid] ){
								this._items.splice(i,1);
								delete this._itemsByGuid[guid];
							}
						}
					}

			};
			return constructor;
		})()
	};
})();
