
var nativeApplication = air.NativeApplication.nativeApplication;

var labels; 
var displayPeriod = "day";
var warningsPresent = false;
var slice = Array.prototype.slice;

var icons = (function(){
    function iconsNamed(name){
        return ["16","32","48","128"].map(function(value){
            return value + "/" + name;
        });
    }
    return {
        count: 0,
        bitmaps:{},
        current:null,
        required:null,
        names:{
            main: iconsNamed("default"),
            "warning-red": iconsNamed("warning-red"),
            "warning-amber": iconsNamed("warning-amber"),
            "warning-yellow": iconsNamed("warning-yellow"),
            error: iconsNamed("nodata"),
            update: iconsNamed("update")
        },
        setCurrent:function(name){
            this.required = name;
            if( typeof this.bitmaps[name] === 'undefined' ){
                this.loadBitmaps(name);
            } else if ( this.bitmaps[name].length > 0 ){
                this.current = name;
                log("setCurrent with bitmaps to " + name);
                nativeApplication.icon.bitmaps = this.bitmaps[name];
            } // else we must be in the process of loading them anyway
        },
        loadBitmaps:function(name){
            var self = this;
            var count = this.names[name].length;
            function decCount(){
                count -= 1;
                if( count === 0){
                    if( self.required == name){
                        self.setCurrent(name);
                    }else{
                        log("skipped setting current as required is now:" + self.required);
                    }
                }
            }
            var store = this.bitmaps[name] = [];
            this.names[name].forEach(function(iconName){
                var iconLoad = new air.Loader();
                iconLoad.contentLoaderInfo.addEventListener(air.Event.COMPLETE, function (event){
                    log(event);
                    store.push(event.target.content.bitmapData);
                    decCount();
                });
                iconLoad.contentLoaderInfo.addEventListener("ioError",function(){
                    error("icon load for ", name, slice(arguments, 0));
                    decCount();
                });
                iconLoad.load(new air.URLRequest("/lib/icons/"+ iconName + ".png"));
            });
        }
    };
})();


function initialise() {
	configureServer();
    var startedTimers = false;
    nativeApplication.autoExit = false;

    nativeApplication.icon.menu = buildSystemTrayMenu();
    if (air.NativeApplication.supportsSystemTrayIcon) {
		nativeApplication.icon.addEventListener("click", showMainWindow);

        // the following attempts to respond to invoke event sent from other instances
        // this is useful so that if the user attempts to launch the application when it is
        // already open in the system tray, the mainWindow will become visible.
        nativeApplication.addEventListener("invoke",function(){
            if( !mainWindow.isVisible()){
                log("invoke - opening mainWindow");
                mainWindow.show();
            }else if( !mainWindow.opening){
                log("invoke - mainWindow.bringToFront");
                mainWindow.bringToFront();
            }else{
                log("invoke - no action");
            }
        });
    }


    if(settings.resetOptions){
        info("Resetting local options");
	    air.EncryptedLocalStore.reset();
    }

    if( air.NativeApplication.supportsStartAtLogin ){
    	try{
    		nativeApplication.startAtLogin = localOptions.boolean("shouldStartAtLogin",true);
    	}
    	catch(error){
    		
    	}
    }

	initNewsFeed();
	
	function init(){
        updateIcon("Met Office", "main");
		getDefaultParams();

	    setupLanguage(getELSParam("moLanguage"), function(){
            if( networkMonitor.available() ){
                updateWarnings(updateForecast);
                if(! startedTimers ){
                    refreshTimers.startTimer(weatherRefreshTimeout, updateForecast);
                    refreshTimers.startTimer(warningsRefreshTimeout, updateWarnings);
                    refreshTimers.startTimer(ms.hours(1), checkForUpdates);
                    startedTimers = true;
                }else{
                    refreshTimers.restart();
                }
            }
	    });		
	}

    nativeApplication.addEventListener("optionsChanged",function(){
    	log("startup reinitialize()");
    	init();
    });
    
	mainWindow.show();
    networkMonitor.up("startup",init);
    networkMonitor.down("startup",function(){
        updateIcon("Met Office : no network connection","error");
        refreshTimers.stop();
    });
}

function checkForUpdates(options){
    options = options || {} ;
    log("checkForUpdates ", options);
    var lastUpdateCheck = localOptions.date("lastUpdateCheck");

    if( options.force
        || lastUpdateCheck === null   // first run
        || Date.now() - lastUpdateCheck.getTime() > ms.hours(24)
        ){
        log("last update was ", lastUpdateCheck);
        if(networkMonitor.available() || options.force ){
            updater.checkForUpdate(function(){
                localOptions.set("lastUpdateCheck", new Date());
                if( updater.updateAvailable() ){
                    updateIcon("A new version is available", "update");
                }
                if( typeof options.success === 'function'){
                    options.success();
                }
            },options.error);
        }
    }
}

function errorHandler(description){
	return function(){
		error("Error : "+ description, slice.call(arguments, 0));
		updateIcon(labels.metOffice+" : "+labels.error.wxnotfound, "main");
	};
}

function updateForecast(){
	if(!warningsPresent ){
		var location = currentLocation.get(); 
		metoffice.forecast.getById({
			id: location.getId(),
			success: function(forecast){
                if( !updater.updateAvailable()){
				    updateDisplayPeriod(forecast,location);
                }
			},
			error: errorHandler("forecast.getById"),
            timeToLive:ms.seconds(15)
		});
	}
}

function updateWarnings(callback) {
    log("updateWarnings()");
    //We need to occasionally check the timezone to pick up daylight saving changes so we
    //do this as part of the occasionaly warnings refresh. The actual api server request 
    //is set to have a 6 hour cache time to live
    metoffice.timezone.applyTimezoneById({id:"Europe/London",timeToLive :ms.hours(6)});
    metoffice.warning.getMaxWarningLevel({
    	timeToLive: ms.seconds(15),
    	success:function(result){
    		warningsPresent = (result != metoffice.warningLevel.GREEN);
//    		warningsPresent = true; // force the warnings override
//            result = metoffice.warningLevel.AMBER;
    		if( !updater.updateAvailable()){
    			if( warningsPresent ){
    				updateIcon("Weather warnings exist", "warning-" + result.toString().toLowerCase());
                }
    			else{
    				updateIcon("Met Office", "main");
	    			if( typeof callback === 'function'){
	                    callback();
	                }
    			}
            }
    	},
    	error:errorHandler("warning.getMaxWarningLevel")
    });
}

function updateDisplayPeriod(forecast, location) {
	metoffice.utility.getDisplayPeriod({
		success: function(displayPeriod) {
			var day = forecast.getForecastDays()[0];
			var values = day.valuesFor(displayPeriod);
			var w = weather(values.getWeather());
			
		    var tooltip = internationalPlace(location.getName()) + 
		    		" - " + internationalDisplayPeriod(displayPeriod) + 
				    ": " + w.toString() + 
				    " " + temperature(values.getTemperature()).toString();
		    updateIcon(tooltip, "main");
		},
		error:errorHandler("utility.getDisplayPeriod")
	});
} 

function updateIcon(text, icon){	
	log("updateIcon", text, icon);
	if( icon ){
		icons.setCurrent(icon);
	}
    if( air.NativeApplication.supportsSystemTrayIcon ){
	    nativeApplication.icon.tooltip = text;
    }
}



function buildSystemTrayMenu(){
	var menu = new air.NativeMenu();
	var configs = [
	            { name: 'Open', handler:function(){mainWindow.show();}},
                { separator:true},
	            { name: 'About', handler:function(){aboutWindow.show();}},
	            { name: 'News', handler:function(){newsWindow.show();}},
	            { name: 'Options', handler:function(){optionsWindow.show();}}
            ];
    if( !air.NativeApplication.supportsDockIcon){
        // if not a mac
        configs.push({ separator:true });
        configs.push({ name:'Exit', handler:onClose});
    }
	configs.forEach(function(config){
		var command = menu.addItem(new air.NativeMenuItem(config.name || '', config.separator || false));
		if( config.handler ){
			command.addEventListener(air.Event.SELECT, config.handler);
		}
	});
	return menu;
}

function initNewsFeed(){
	var readItems = localOptions.object("moNewsReadItems",[]), 
		firstRun = localOptions.get("moNewsReadItems") === null;
	
	function saveReadItems(){
		localOptions.set("moNewsReadItems", readItems);
	}
	
	window.newsFeed = new rss.Feed({
		url:settings.newsFeed,
//		url:"http://localhost:8000/rssText.xml",
		readGuids:readItems,
		'itemRead':function(item){
			//log("itemRead:" + item.guid, item);
			readItems.push(item.guid);
			saveReadItems();
		},
		'unReadItemCountChanged':function(feed, count, oldCount){
			if( !firstRun ){
				if( count == 0 || oldCount == 0 ){
					// either the count is now zero or was zero
					nativeApplication.dispatchEvent(new air.Event(
						count > 0 ?"newNewsItemsAvailable":"noNewsItemsAvailable"
					));
				}
			}
		}
	});

    // only have to worry about the up event as teh global down will stop the timer
    networkMonitor.up("startup",function(){
        // wait for 30 seconds and force the first load.
        setTimeout(function(){
            newsFeed.load({success:function(feed){
                if( firstRun ){
                    info("initNewsFeed:firstRun marking All items as read");
                    // mark all the items as read in the news feed if this is first run of
                    // the application
                    feed.items(function(item){item.markRead();});
                    firstRun = false;
                }
            }});
        }, ms.seconds(30));

        if( !newsFeed.refreshTimer ){
            newsFeed.refreshTimer = refreshTimers.startTimer(ms.minutes(60), function(){
                info("refreshing the newsFeed");
                newsFeed.load({success:function(feed){
                    var knownGuids = {};
                    feed.items(function(item){knownGuids[item.guid] = true;});

                    var c = 0;
                    for( var i = readItems.length -1 ; i >= 0 ; i-- ){
                        if( !knownGuids[readItems[i]]){
                            readItems.splice(i,1);
                            c++;
                        }
                    }
                    log("Trimmed readItems :", c, readItems);
                    saveReadItems();
                }});
            });
        }
    });
}

function showMainWindow(){
    mainWindow.show({showWarnings:warningsPresent});
    if( updater.updateAvailable()){
        updateWindow.show();
    }
}